import numpy as np
import rpy2.rlike.container as rlc
import rpy2.robjects.numpy2ri
from rpy2.robjects import DataFrame, StrVector, Formula
from rpy2.robjects.packages import importr

from lib5c.util.system import check_outdir

grdevices = importr('grDevices')
deseq2 = importr('DESeq2')
rpy2.robjects.numpy2ri.activate()


def compute_dispersions(obs_arrays, min_obs=2, plot_outfile=None):
    obs_data = np.floor(np.stack(obs_arrays))
    idx = np.all(obs_data >= min_obs, axis=0)
    countdata_df = DataFrame(rlc.OrdDict([('rep%i' % i, obs_data[i][idx])
                                          for i in range(len(obs_arrays))]))
    coldata_df = DataFrame({'condition': StrVector(['cond'] * len(obs_arrays))})
    coldata_df.rownames = StrVector(['rep%i' % i
                                     for i in range(len(obs_arrays))])
    dds = deseq2.DESeqDataSetFromMatrix(
        countData=countdata_df, colData=coldata_df, design=Formula('~ 1'))
    dds = deseq2.estimateSizeFactors_DESeqDataSet(dds)
    dds = deseq2.estimateDispersions_DESeqDataSet(dds, fitType='local')
    disp = np.tile(np.nan, obs_data[0].shape)
    disp[idx] = np.asarray(deseq2.dispersions_DESeqDataSet(dds))
    if plot_outfile:
        check_outdir(plot_outfile)
        grdevices.png(file=plot_outfile, height=8, width=8, units='in', res=300)
        deseq2.plotDispEsts_DESeqDataSet(dds)
        grdevices.dev_off()
    return disp


def compute_variance(disp, obs_arrays):
    obs_data = np.floor(np.stack(obs_arrays))
    exp = np.mean(obs_data, axis=0)
    return exp + disp*exp**2
