import os

from lib5c.plotters.matrix import plot_matrix

from hiclite.util.filenames import format_outfile
from hiclite.util.sparse_matrix import SparseContactMatrix as SCM


def process_obs(args, config, data):
    outfile = format_outfile(config, args.rep, args.chrom, 'obs')
    if os.path.exists(outfile):
        print('obs already exists, skipping')
        data['obs'] = SCM.load(outfile)
        data['current'] = data['obs']
        return

    print('making obs')
    print(' constructing SparseContactMatrix')
    obs_scm = SCM(
        data['current'],
        max_range=config.getint('obs_params', 'max_range')
    )

    print(' saving and plotting obs')
    obs_scm.save(outfile)
    if config.getboolean('matrix', 'plot'):
        plot_matrix(obs_scm, log=True, pseudocount=1, outfile=format_outfile(
            config, args.rep, args.chrom, 'obs', ext='png'))

    data['obs'] = obs_scm
    data['current'] = data['obs']
