import os

import numpy as np
from statsmodels.stats.multitest import multipletests

from hiclite.util.sparse_matrix import SparseContactMatrix as SCM
from hiclite.util.filenames import format_outfile
from hiclite.util.lambda_chunking import compute_lamdba_chunked_qvalues


def process_qvalues(args, config):
    print('making qvalues')
    chroms = zip(*config.items('chroms'))[0]
    pvalue_scms = {
        chrom: SCM.load(format_outfile(config, args.rep, chrom, 'pvalues'))
        for chrom in chroms
    }
    outfiles = {chrom: format_outfile(config, args.rep, chrom, 'qvalues')
                for chrom in chroms}
    if config.getboolean('qvalues_params', 'lambda_chunk'):
        exp_scms = {
            chrom: SCM.load(format_outfile(config, args.rep, chrom, 'exp'))
            for chrom in chroms
        }
        if config.getboolean('var_params', 'deconvolute'):
            print(' deconvoluting')
            for chrom in chroms:
                exp_scms[chrom] = exp_scms[chrom].deconvolute(
                    np.loadtxt(format_outfile(config, args.rep, chrom, 'bias')))
        print(' lambda chunking')
        plot_pattern = os.path.split(outfiles[chroms[0]])[0] + '/%s_%s.png' \
            if config.getboolean('qvalues_params', 'plot_chunks') else None
        pvalue_matrices = {chrom: pvalue_scms[chrom].data for chrom in chroms}
        exp_matrices = {chrom: exp_scms[chrom].data for chrom in chroms}
        qvalue_matrices = compute_lamdba_chunked_qvalues(
            pvalue_matrices, exp_matrices, plot_pattern=plot_pattern)
        print(' saving qvalues')
        for chrom in chroms:
            pvalue_scms[chrom].data = qvalue_matrices[chrom]
            pvalue_scms[chrom].save(outfiles[chrom])
    else:
        finite_idx = {chrom: np.isfinite(pvalue_scms[chrom].data)
                      for chrom in chroms}
        slice_bounds = np.concatenate([[0], np.cumsum([finite_idx[chrom].sum()
                                                       for chrom in chroms])])
        qvalues = multipletests(
            np.concatenate([pvalue_scms[chrom].data[finite_idx[chrom]]
                            for chrom in chroms]),
            0.1, method='fdr_bh')[1]
        for i, chrom in enumerate(chroms):
            pvalue_scms[chrom].data[finite_idx[chrom]] = \
                qvalues[slice_bounds[i]:slice_bounds[i+1]]
            pvalue_scms[chrom].save(outfiles[chrom])
