import ConfigParser

import numpy as np

from lib5c.plotters.curve_fits import plot_fit

from hiclite.util.cross_rep_variance import estimate_pixel_wise_variance
from hiclite.util.filenames import format_outfile
from hiclite.util.sparse_matrix import SparseContactMatrix as SCM
from hiclite.toolbox import subcommand, argument


@subcommand('plot_crv', 'visualize cross-replicate variance', [
    argument('rep'),
    argument('chrom'),
    argument('--logx', action='store_true'),
    argument('-S', '--scatter', action='store_true'),
    argument('--base_outdir', default='heatmaps'),
    argument('--config', default='config.cfg')
])
def main(args):
    print('parsing config')
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.optionxform = str
    config.read(args.config)

    print('loading data')
    obs_scms = [SCM.load(format_outfile(config, rep, args.chrom, 'obs'))
                for rep in zip(*config.items('reps'))[0]]
    exp_scm = SCM.load(format_outfile(config, args.rep, args.chrom, 'exp'))
    var_scm = SCM.load(format_outfile(config, args.rep, args.chrom, 'var'))

    disp_mle, mean_mle, var_mle, exp, samp_mean, samp_var, idx = \
        estimate_pixel_wise_variance(
            [scm.data for scm in obs_scms], exp_scm.data,
            min_obs=config.getfloat('var_params', 'min_obs'),
            min_dist=config.getint('var_params', 'min_dist'))

    print('plotting CRV')
    outfile = '%s/crv_%s_%s%s%s.png' % (args.base_outdir, args.rep, args.chrom,
                                        '_logx' if args.logx else '',
                                        '_scatter' if args.scatter else '')
    sort_idx = np.argsort(exp)
    plot_fit(exp[sort_idx], samp_var[sort_idx], var_scm.data[idx][sort_idx],
             logx=args.logx, logy=True, hexbin=not args.scatter,
             xlabel='expected value', ylabel='variance', outfile=outfile)
