import ConfigParser

import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
from ast import literal_eval
import numpy as np
import subprocess

from lib5c.util.plotting import plotter
from lib5c.util.system import check_outdir
from hiclite.util.filenames import format_outfile
from hiclite.util.sparse_matrix import SparseContactMatrix
from hiclite.toolbox import subcommand, argument

def make_histogram_cmd(args, config,chrom=None):
    if chrom is None:
        chrom = args.chrom
    if config.getboolean('scheduler', 'bsub'):
        prefix = 'bsub -M %s -n %s ' % (
            config.get('scheduler', 'process_memory'),
            config.get('scheduler', 'process_cores'))
    else:
        prefix = ''
    return '%shiclite plot_distribution_across_reps histogram --stage %s --chrom %s --config %s' % \
        (prefix, args.stage, chrom, args.config)

def histogram(stage,chrom,config):

    outfile = 'plots/all_reps_%s_%s_histograms.png'%(chrom,stage)

    reps = [config.items('reps')[i][0] for i in range(len(config.items('reps')))]

    labels = {rep: rep for rep in reps}

    levels = {label: label for label in labels.values()}

    unique_levels = list(set(levels.values()))
    palette = sns.color_palette('husl', len(unique_levels))
    colors = {unique_levels[i]: palette[i]
                  for i in range(len(unique_levels))}

    data = {rep: SparseContactMatrix.load(format_outfile(config, rep,
                    chrom, stage)).flatten() for rep in reps}
    
    nonzero_data = {rep: data[rep][data[rep] > 0] for rep in reps}

    for rep in reps:
        plt.hist(
            np.log2(nonzero_data[rep] + 1),
            color=colors[levels[labels[rep]]],
            label=labels[rep],
            histtype='step',
            density=True,
            bins=100
        )
    plt.legend()
    plt.ylabel('Probability density')
    plt.xlabel('log2(counts + 1)')

    legend_labels = list(set(levels.values()))
    legend_handles = [mpatches.Patch(color=colors[l]) for l in legend_labels]
    plt.legend(legend_handles, legend_labels, scatterpoints=1, loc='upper left',
               bbox_to_anchor=(1, 1.05))

    plt.savefig(outfile,dpi=800)

@subcommand('plot_distribution_across_reps', 'plot distribution across reps for ' \
                'quality checking (QC) purposes', [
    argument('action', choices=['histogram']),
    argument('--stage', choices=['input','raw','filtered','balanced',\
                                'obs','exp','pvalues']),
    argument('--chrom', default='all'),
    argument('--config', default='config.cfg')
])
def main(args):

    print('parsing config')
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.optionxform = str
    config.read(args.config)
    
    if args.action == 'histogram':
        if args.chrom == 'all':
            for chrom, _ in config.items('chroms'):
                cmd = make_histogram_cmd(args, config, chrom)
                print(cmd)
                subprocess.call(cmd,shell=True)
        else:
            histogram(args.stage,args.chrom, config)
    
if __name__ == '__main__':
    main()
