import ConfigParser
import subprocess

from lib5c.parsers.bed import load_features
from lib5c.util.bed import flatten_features

from hiclite.toolbox import subcommand, argument


@subcommand('plot_locus_set', 'plot heatmaps around all loci in a bedfile', [
    argument('locus_set_name'),
    argument('rep'),
    argument('--outdir'),
    argument('--config', default='config.cfg')
])
def main(args):
    print('parsing config')
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.optionxform = str
    config.read(args.config)

    # resolve args
    outdir = config.get('heatmap', 'outdir') if args.outdir is None \
        else args.outdir

    loci = flatten_features(load_features(
        config.get('locus_sets', args.locus_set_name)))
    for locus in loci:
        grange = '%s:%i-%i' % (locus['chrom'], locus['start'], locus['end'])

        # on-diagonal, 3 Mb window
        prefix = 'bsub ' if config.getboolean('scheduler', 'bsub') else ''
        cmd = '%s hiclite zoom %s %s --outdir %s --config %s --name %s' \
              % (prefix, grange, args.rep, outdir, args.config, locus['id'])
        print cmd
        subprocess.call(cmd, shell=True)


if __name__ == '__main__':
    main()
