import ConfigParser

import pandas as pd
import seaborn as sns

from lib5c.util.bed import parse_feature_from_string, get_mid_to_mid_distance
from lib5c.util.plotting import plotter

from hiclite.util.filenames import format_outfile
from hiclite.toolbox import subcommand, argument


@plotter
def plot_distances(distances, **kwargs):
    sns.distplot(distances, kde=False)


def compute_distance(row):
    anchors = map(parse_feature_from_string, row.name.split('_'))
    return get_mid_to_mid_distance(*anchors)


@subcommand('plot_loop_dists', 'plot histogram of called loop distances', [
    argument('rep'),
    argument('--base_outdir', default='heatmaps'),
    argument('--config', default='config.cfg')
])
def main(args):
    print('parsing config')
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.optionxform = str
    config.read(args.config)

    # load dataframe
    df = pd.read_csv(format_outfile(
        config, args.rep, 'all', 'loops', ext='tsv'), sep='\t', index_col=0)

    # compute distances
    df['distance'] = df.apply(compute_distance, axis=1)

    # plot distances
    plot_distances(df['distance'], xlim=[0, 3e6], ylabel='number of loops',
                   outfile='%s/%s_distances.png' % (args.base_outdir, args.rep))


if __name__ == '__main__':
    main()
