import ConfigParser

import numpy as np
import matplotlib.pyplot as plt

from lib5c.util.plotting import plotter

from hiclite.util.mvr import fit_mvr
from hiclite.util.filenames import format_outfile
from hiclite.util.sparse_matrix import SparseContactMatrix as SCM
from hiclite.toolbox import subcommand, argument


@plotter
def plot_mvr(obs_scm, exp_scm, dpi=800, **kwargs):
    var, xs, ys, curve_xs, curve_ys = fit_mvr(
        obs_scm.data, exp_scm.data, return_intermediates=True, return_curve=100)
    plt.hexbin(xs, ys, xscale='log', yscale='log', bins='log',
               cmap='Blues', linewidths=0.1)
    plt.plot(curve_xs, curve_ys, color='r')


@subcommand('plot_mvr', 'visualize mean-variance relationship', [
    argument('rep'),
    argument('chrom'),
    argument('--base_outdir', default='heatmaps'),
    argument('--config', default='config.cfg')
])
def main(args):
    print('parsing config')
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.optionxform = str
    config.read(args.config)

    print('loading data')
    obs_scm = SCM.load(format_outfile(config, args.rep, args.chrom, 'obs'))
    exp_scm = SCM.load(format_outfile(config, args.rep, args.chrom, 'exp'))

    if config.getboolean('var_params', 'deconvolute'):
        print('deconvoluting')
        bias = np.loadtxt(format_outfile(config, args.rep, args.chrom, 'bias'))
        obs_scm = obs_scm.deconvolute(bias)
        exp_scm = exp_scm.deconvolute(bias)

    print('plotting MVR')
    outfile = '%s/mvr_%s_%s.png' % (args.base_outdir, args.rep, args.chrom)
    plot_mvr(obs_scm, exp_scm, outfile=outfile)
