import ConfigParser
import subprocess

import pandas as pd

from hiclite.util.filenames import format_outfile
from hiclite.toolbox import subcommand, argument


@subcommand('plot_top_loops', 'plot heatmaps around most significant loops', [
    argument('rep'),
    argument('--by', default='pvalue'),
    argument('--descending', action='store_true'),
    argument('-n', default=5, type=int),
    argument('--size_on', default=200, type=int),
    argument('--size_off', default=30, type=int),
    argument('--outdir'),
    argument('--config', default='config.cfg')
])
def main(args):
    print('parsing config')
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.optionxform = str
    config.read(args.config)

    # resolve args
    outdir = config.get('heatmap', 'outdir') if args.outdir is None \
        else args.outdir

    df = pd.read_csv(format_outfile(
        config, args.rep, 'all', 'loops', ext='tsv'), sep='\t', index_col=0)
    df.dropna(subset=[args.by], inplace=True)
    df.sort_values(by=args.by, ascending=not args.descending, inplace=True)
    for i in range(args.n):
        loop_id = df.index[i]
        grange_y, grange_x = loop_id.split('_')
        prefix = 'bsub ' if config.getboolean('scheduler', 'bsub') else ''

        # on-diagonal, 2 Mb window
        cmd = '%shiclite zoom %s -y %s %s --diag on --size %s --outdir %s ' \
              '--config %s' \
              % (prefix, grange_x, grange_y, args.rep, args.size_on, outdir,
                 args.config)
        print cmd
        subprocess.call(cmd, shell=True)

        # off-diagonal, 300 kb window
        cmd = '%shiclite zoom %s -y %s %s --diag off --size %s --outdir %s ' \
              '--config %s' \
              % (prefix, grange_x, grange_y, args.rep, args.size_off, outdir,
                 args.config)
        print cmd
        subprocess.call(cmd, shell=True)


if __name__ == '__main__':
    main()
