import os
import subprocess

from hiclite.util.filenames import format_outfile
from hiclite.util.config import parse_config
from hiclite.toolbox import subcommand, argument
from lib5c.util.system import shell_quote


@subcommand('view', 'open a chromosome in hiczoom', [
    argument('rep'),
    argument('chrom'),
    argument('--track'),
    argument('-l', '--loops_override', type=str),
    argument('--stage', choices=['input', 'raw', 'filtered', 'balanced', 'obs',
                                 'exp', 'pvalues'], default='obs'),
    argument('--cmap'),
    argument('-p', '--port', default=8000),
    argument('--config', default='config.cfg')
])
def main(args):
    try:
        import hiczoom
    except ImportError, e:
        print('hiczoom not installed, install it and try again')
        raise e

    print('parsing config')
    configs = [parse_config(configfile)
               for configfile in args.config.split(',')]
    config = configs[0]
    res = config.getint('assemble_params', 'resolution')
    genome = config.get('matrix', 'genome')
    loops = []
    if args.loops_override:
        for f in args.loops_override.split(','):
            if os.path.exists(f):
                loops.append(f)
    else:
        for c in configs:
            try:
                f = format_outfile(c, args.rep, args.chrom, 'loops', ext='tsv')
                if os.path.exists(f):
                    loops.append(f)
            except ValueError:
                pass
    if not loops:
        loops = None
    matrix = format_outfile(config, args.rep, args.chrom, args.stage)
    track = config.get('tracks', args.track) if args.track else None
    vmax = config.getfloat('heatmap', 'vmax')
    vmin = -vmax if args.stage == 'pvalues' else 0
    if args.cmap is None:
        if args.stage == 'pvalues':
            cmap = 'seismic'
        else:
            cmap = 'Reds'
    else:
        cmap = args.cmap

    cmd = 'hiczoom -p %s -c %s --vmin %s --vmax %s %s %s %s %s %s %s' % \
        (args.port, cmap, vmin, vmax,
         '-l %s' % shell_quote(','.join(loops)) if loops else '',
         '-b %s' % track if track else '', matrix, genome, args.chrom, res)
    if args.stage in ['pvalues', 'qvalues']:
        cmd += ' -Z'
    print(cmd)
    subprocess.call(cmd, shell=True)


if __name__ == '__main__':
    main()
