import numpy as np

from lib5c.util.lowess import lowess_fit

MIN_DISP = 1e-8
MIN_VAR = 1e-8


def disp_to_var(disp, exp):
    mean = np.log(exp) - disp / 2  # convert exp to mean of log counts
    return (np.exp(disp) - 1) * np.exp(2 * mean + disp)


def estimate_pixel_wise_variance(obs_arrays, exp_array, min_obs=2, min_dist=4):
    obs_data = np.array(obs_arrays)
    dist_data = np.tile(np.arange(exp_array.shape[0]),
                        (exp_array.shape[1], 1)).T
    idx = np.isfinite(obs_data).all(axis=0) & \
        np.all(obs_data >= min_obs, axis=0) & \
        np.isfinite(exp_array) & (exp_array > 0) & (dist_data >= min_dist)
    exp = exp_array[idx]
    matrix = np.array([obs_data[i][idx] for i in range(len(obs_data))])
    samp_mean = np.mean(matrix, axis=0)
    samp_var = np.var(matrix, axis=0)
    idx_od = samp_var > MIN_VAR
    samp_var[~idx_od] = MIN_VAR
    log_matrix = np.log(matrix)
    mu_mle = np.mean(log_matrix, axis=0)
    disp_mle = np.var(log_matrix, axis=0)
    idx_od = disp_mle > MIN_DISP
    disp_mle[~idx_od] = MIN_DISP
    mean_mle = np.exp(mu_mle + disp_mle / 2)
    var_mle = (np.exp(disp_mle) - 1) * np.exp(2 * mu_mle + disp_mle)
    return disp_mle, mean_mle, var_mle, exp, samp_mean, samp_var, idx


def fit_cross_rep_variance(obs_arrays, exp_array, min_obs=2, min_exp=0.5,
                           min_dist=4):
    disp_mle, mean_mle, var_mle, exp, samp_mean, samp_var, idx = \
        estimate_pixel_wise_variance(
            obs_arrays, exp_array, min_obs=min_obs, min_dist=min_dist)

    print(' fitting mvr')
    fit = lowess_fit(exp, disp_mle, logx=True)

    from lib5c.plotters.curve_fits import plot_fit
    plot_fit(exp, samp_var, disp_to_var(fit(exp), exp), logx=False, logy=True,
             hexbin=True, outfile='test/mvr_semilog.png')

    print(' saving var')
    exp_array = np.where(exp_array < min_exp, min_exp, exp_array)
    exp = exp_array[idx]
    var_data = np.tile(np.nan, exp_array.shape)
    var_data[idx] = disp_to_var(fit(exp), exp)
    return var_data
